@@ -22,6 +22,7 @@
 #include "financial.h"
 #include "currency.h"
 #include "mp-serializer.h"
+#include "units.h"
 
 enum {
     PROP_0,
@@ -56,7 +57,8 @@ struct MathButtonsPrivate
 
     GtkWidget *convert_from_combo;
     GtkWidget *convert_to_combo;
-    gchar *previous_ans;
+    MpSerializer *units_serializer;
+    GtkWidget *convert_result_label;
 
     GtkWidget *base_combo;
     GtkWidget *base_label;
@@ -65,6 +67,7 @@ struct MathButtonsPrivate
 
     GtkWidget *source_currency_combo;
     GtkWidget *target_currency_combo;
+    MpSerializer *currency_serializer;
     GtkWidget *currency_label;
 
     GtkWidget *character_code_dialog;
@@ -313,12 +316,12 @@ static char *finc_dialog_fields[][5] = {
 
 #define MAX_UNITS 20
 struct Unit {
-    char* ui_name;
-    char* internal_name;
+    char *ui_name;
+    char *internal_name;
 };
 
 struct UnitCategory {
-    char* name;
+    char *name;
     struct Unit units[MAX_UNITS];
 };
 
@@ -337,7 +340,7 @@ static struct UnitCategory categories[] = {
                           /* Length unit */
                           {N_("Light Years"), "lightyears"},
                           /* Length unit */
-                          {N_("Astronomical Unit"), "au"},
+                          {N_("Astronomical Units"), "au"},
                           /* Length unit */
                           {N_("Nautical Miles"), "nm"},
                           /* Length unit */
@@ -428,12 +431,12 @@ static struct UnitCategory categories[] = {
 MathButtons *
 math_buttons_new(MathEquation *equation)
 {
-    return g_object_new (math_buttons_get_type(), "equation", equation, NULL);
+    return g_object_new(math_buttons_get_type(), "equation", equation, NULL);
 }
 
 
 static void
-set_tint (GtkWidget *widget, GdkColor *tint, gint alpha)
+set_tint(GtkWidget *widget, GdkColor *tint, gint alpha)
 {
     GtkStyle *style;
     int j;
@@ -593,8 +596,8 @@ update_currency_label(MathButtons *buttons)
         const char *source_symbol, *target_symbol;
         int i;
 
-        mp_serializer_to_specific_string(&x, 10, 2, false, true, &input_text);
-        mp_serializer_to_specific_string(&value, 10, 2, false, true, &output_text);
+        input_text = mp_serializer_to_string(buttons->priv->currency_serializer, &x);
+        output_text = mp_serializer_to_string(buttons->priv->currency_serializer, &value);
 
         for (i = 0; strcmp(math_equation_get_source_currency(buttons->priv->equation), currency_names[i].short_name) != 0; i++);
         source_symbol = currency_names[i].symbol;
@@ -616,11 +619,48 @@ update_currency_label(MathButtons *buttons)
 }
 
 
+static void
+update_conversion_bar(MathButtons *buttons)
+{
+    MPNumber x, z;
+    gboolean enabled;
+    gchar *label;
+    const gchar *source_units, *target_units;
+    char *source_value, *target_value;
+
+    if (!buttons->priv->convert_result_label)
+        return;
+
+    enabled = math_equation_get_number(buttons->priv->equation, &x);
+
+    source_units = math_equation_get_source_units(buttons->priv->equation);
+    target_units = math_equation_get_target_units(buttons->priv->equation);
+    if (!source_units || !target_units)
+        enabled = FALSE;
+    else if (!units_convert(&x, source_units, target_units, &z))
+        enabled = FALSE;
+
+    gtk_widget_set_sensitive(buttons->priv->convert_result_label, enabled);
+    if (!enabled)
+        return;
+
+    source_value = mp_serializer_to_string(buttons->priv->units_serializer, &x);
+    target_value = mp_serializer_to_string(buttons->priv->units_serializer, &z);
+
+    label = g_strdup_printf("%s %s = %s %s", source_value, source_units, target_value, target_units);
+    gtk_label_set_text(GTK_LABEL(buttons->priv->convert_result_label), label);
+    g_free(source_value);
+    g_free(target_value);
+    g_free(label);
+}
+
+
 static void
 display_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons *buttons)
 {
     update_currency_label(buttons);
     update_bit_panel(buttons);
+    update_conversion_bar(buttons);
 }
 
 
@@ -628,23 +668,57 @@ static void
 convert_from_combobox_changed_cb(GtkWidget *combo, MathButtons *buttons)
 {
     GtkTreeModel *model;
-    GtkTreeIter iter, unittype;
-    int typeindex, i;
+    GtkTreeIter iter;
+    int typeindex, unitindex, i;
 
     model = gtk_combo_box_get_model(GTK_COMBO_BOX(combo));
     gtk_combo_box_get_active_iter(GTK_COMBO_BOX(combo), &iter);
+    gtk_tree_model_get(model, &iter, 1, &typeindex, 2, &unitindex, -1);
 
-    if (!gtk_tree_model_iter_parent(model, &unittype, &iter))
-        return;
-
-    gtk_tree_model_get(model, &unittype, 1, &typeindex, -1);
-
-    model = GTK_TREE_MODEL(gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT));
+    model = GTK_TREE_MODEL(gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT));
     for (i = 0; categories[typeindex].units[i].ui_name != NULL; i++) {
+        if (i == unitindex)
+            continue;
         gtk_list_store_append(GTK_LIST_STORE(model), &iter);
-        gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, categories[typeindex].units[i].ui_name, 1, i, -1);
+        gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, categories[typeindex].units[i].ui_name, 1, typeindex, 2, i, -1);
     }
     gtk_combo_box_set_model(GTK_COMBO_BOX(buttons->priv->convert_to_combo), model);
+
+    math_equation_set_source_units(buttons->priv->equation, categories[typeindex].units[unitindex].internal_name);
+
+    gtk_combo_box_set_active(GTK_COMBO_BOX(buttons->priv->convert_to_combo), 0);
+}
+
+
+static void
+source_units_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons *buttons)
+{
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+
+    model = gtk_combo_box_get_model(GTK_COMBO_BOX(buttons->priv->convert_from_combo));
+    if (!gtk_tree_model_get_iter_first(model, &iter))
+        return;
+    do
+    {
+        GtkTreeIter child_iter;
+
+        if (gtk_tree_model_iter_children(model, &child_iter, &iter))
+        {
+            do
+            {
+                gint i, j;
+
+                gtk_tree_model_get(model, &child_iter, 1, &i, 2, &j, -1);
+                if (strcmp(categories[i].units[j].internal_name, math_equation_get_source_units(equation)) == 0)
+                {
+                    gtk_combo_box_set_active_iter(GTK_COMBO_BOX(buttons->priv->convert_from_combo), &child_iter);
+                    update_conversion_bar(buttons);
+                    return;
+                }             
+            } while (gtk_tree_model_iter_next(model, &child_iter));
+        }
+    } while (gtk_tree_model_iter_next(model, &iter));
 }
 
 
@@ -652,52 +726,40 @@ static void
 convert_to_combobox_changed_cb(GtkWidget *combo, MathButtons *buttons)
 {
     GtkTreeModel *model;
-    GtkTreeIter iter, catiter;
-    gchar *from, *to, *display;
-    int category, fromindex, toindex;
+    GtkTreeIter iter;
+    int category, toindex;
 
     model = gtk_combo_box_get_model(GTK_COMBO_BOX(combo));
     gtk_combo_box_get_active_iter(GTK_COMBO_BOX(combo), &iter);
-    gtk_tree_model_get(model, &iter, 1, &toindex, -1);
-
-    model = gtk_combo_box_get_model(GTK_COMBO_BOX(buttons->priv->convert_from_combo));
-    gtk_combo_box_get_active_iter(GTK_COMBO_BOX(buttons->priv->convert_from_combo), &iter);
-    if (!gtk_tree_model_iter_parent(model, &catiter, &iter))
-        return;
-
-    gtk_tree_model_get(model, &catiter, 1, &category, -1);
-    gtk_tree_model_get(model, &iter, 1, &fromindex, -1);
-
-    from = categories[category].units[fromindex].internal_name;
-    to = categories[category].units[toindex].internal_name;
-
-    if (buttons->priv->previous_ans == NULL) {
-        display = math_equation_get_display(buttons->priv->equation);
-        buttons->priv->previous_ans = display;
-    }
-    else {
-        display = buttons->priv->previous_ans;
-    }
-    math_equation_set(buttons->priv->equation, g_strdup_printf("%s %s in %s", display, from, to));
+    gtk_tree_model_get(model, &iter, 1, &category, 2, &toindex, -1);
+    math_equation_set_target_units(buttons->priv->equation, categories[category].units[toindex].internal_name);
 }
 
+
 static void
-update_conversion_combos(MathEquation *equation, MathButtons *buttons)
+target_units_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons *buttons)
 {
     GtkTreeModel *model;
     GtkTreeIter iter;
 
-    model = gtk_combo_box_get_model(GTK_COMBO_BOX(buttons->priv->convert_from_combo));
-    gtk_tree_model_get_iter_first(model, &iter);
-    gtk_combo_box_set_active_iter(GTK_COMBO_BOX(buttons->priv->convert_from_combo), &iter);
+    model = gtk_combo_box_get_model(GTK_COMBO_BOX(buttons->priv->convert_to_combo));
+    if (!gtk_tree_model_get_iter_first(model, &iter))
+        return;
+    do
+    {
+        gint i, j;
 
-    model = GTK_TREE_MODEL(gtk_list_store_new(1, G_TYPE_STRING));
-    gtk_combo_box_set_model(GTK_COMBO_BOX(buttons->priv->convert_to_combo), model);
-
-    g_free(buttons->priv->previous_ans);
-    buttons->priv->previous_ans = NULL;
+        gtk_tree_model_get(model, &iter, 1, &i, 2, &j, -1);
+        if (strcmp(categories[i].units[j].internal_name, math_equation_get_target_units(equation)) == 0)
+        {
+            gtk_combo_box_set_active_iter(GTK_COMBO_BOX(buttons->priv->convert_to_combo), &iter);
+            update_conversion_bar(buttons);
+            return;
+        }
+    } while (gtk_tree_model_iter_next(model, &iter));
 }
 
+
 static void
 base_combobox_changed_cb(GtkWidget *combo, MathButtons *buttons)
 {
@@ -754,7 +816,7 @@ source_currency_combo_changed_cb(GtkWidget *combo, MathButtons *buttons)
     gtk_tree_model_get(model, &iter, 0, &value, -1);
 
     math_equation_set_source_currency(buttons->priv->equation, value);
-    g_free (value);
+    g_free(value);
 }
 
 
@@ -776,8 +838,8 @@ source_currency_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons
         gboolean matched;
 
         gtk_tree_model_get(model, &iter, 0, &v, -1);
-        matched = strcmp (math_equation_get_source_currency(buttons->priv->equation), v) == 0;
-        g_free (v);
+        matched = strcmp(math_equation_get_source_currency(buttons->priv->equation), v) == 0;
+        g_free(v);
         if (matched)
             break;
         valid = gtk_tree_model_iter_next(model, &iter);
@@ -802,7 +864,7 @@ target_currency_combo_changed_cb(GtkWidget *combo, MathButtons *buttons)
     gtk_tree_model_get(model, &iter, 0, &value, -1);
 
     math_equation_set_target_currency(buttons->priv->equation, value);
-    g_free (value);
+    g_free(value);
 }
 
 
@@ -824,8 +886,8 @@ target_currency_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons
         gboolean matched;
 
         gtk_tree_model_get(model, &iter, 0, &v, -1);
-        matched = strcmp (math_equation_get_target_currency(buttons->priv->equation), v) == 0;
-        g_free (v);
+        matched = strcmp(math_equation_get_target_currency(buttons->priv->equation), v) == 0;
+        g_free(v);
         if (matched)
             break;
         valid = gtk_tree_model_iter_next(model, &iter);
@@ -838,6 +900,17 @@ target_currency_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons
 }
 
 
+static void
+convert_from_cell_data_func(GtkCellLayout   *cell_layout,
+                            GtkCellRenderer *cell,
+                            GtkTreeModel    *tree_model,
+                            GtkTreeIter     *iter,
+                            gpointer         data)
+{
+    g_object_set(cell, "sensitive", !gtk_tree_model_iter_has_child(tree_model, iter), NULL);
+}
+
+
 static GtkWidget *
 load_mode(MathButtons *buttons, ButtonMode mode)
 {
@@ -853,6 +926,7 @@ load_mode(MathButtons *buttons, ButtonMode mode)
     GError *error = NULL;
 
     switch (mode) {
+    default:
     case BASIC:
         builder_ptr = &buttons->priv->basic_ui;
         builder_file = UI_BASIC_FILE;
@@ -906,7 +980,7 @@ load_mode(MathButtons *buttons, ButtonMode mode)
         if (button_data[i].tooltip)
             gtk_widget_set_tooltip_text(button, _(button_data[i].tooltip));
       
-        atk_object_set_name (gtk_widget_get_accessible (button), button_data[i].widget_name);
+        atk_object_set_name(gtk_widget_get_accessible(button), button_data[i].widget_name);
 
         switch (button_data[i].class) {
         case NUMBER:
@@ -940,16 +1014,25 @@ load_mode(MathButtons *buttons, ButtonMode mode)
         name = g_strdup_printf("calc_%d_button", i);
         button = GET_WIDGET(builder, name);
         if (button) {
+            gchar buffer[7];
+            gint len;
+
             g_object_set_data(G_OBJECT(button), "calc_digit", GINT_TO_POINTER(i));
             set_tint(button, &buttons->priv->color_numbers, 1);
-            gtk_button_set_label(GTK_BUTTON(button), math_equation_get_digit_text(buttons->priv->equation, i));
+            len = g_unichar_to_utf8(math_equation_get_digit_text(buttons->priv->equation, i), buffer);
+            buffer[len] = '\0';
+            gtk_button_set_label(GTK_BUTTON(button), buffer);
         }
         g_free(name);
     }
     widget = GET_WIDGET(builder, "calc_numeric_point_button");
     if (widget) {
         MpSerializer *serializer = math_equation_get_serializer(buttons->priv->equation);
-        gtk_button_set_label(GTK_BUTTON(widget), mp_serializer_get_numeric_point_text(serializer));
+        gchar buffer[7];
+        gint len;
+        len = g_unichar_to_utf8(mp_serializer_get_radix(serializer), buffer);
+        buffer[len] = '\0';
+        gtk_button_set_label(GTK_BUTTON(widget), buffer);
     }
   
     widget = GET_WIDGET(builder, "calc_superscript_button");
@@ -971,10 +1054,11 @@ load_mode(MathButtons *buttons, ButtonMode mode)
         GtkCellRenderer *renderer;
         int i, j;
 
-//        buttons->priv->angle_label = GET_WIDGET(builder, "angle_label");
+        buttons->priv->convert_result_label = GET_WIDGET(builder, "convert_result_label");
+        buttons->priv->units_serializer = mp_serializer_new(MP_DISPLAY_FORMAT_AUTOMATIC, 10, 2);
 
         buttons->priv->convert_from_combo = GET_WIDGET(builder, "convert_from_combo");
-        from_model = gtk_tree_store_new(2, G_TYPE_STRING, G_TYPE_INT);
+        from_model = gtk_tree_store_new(3, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT);
         gtk_combo_box_set_model(GTK_COMBO_BOX(buttons->priv->convert_from_combo), GTK_TREE_MODEL(from_model));
 
         for (i = 0; i < sizeof(categories) / sizeof(categories[0]); i++) {
@@ -982,12 +1066,16 @@ load_mode(MathButtons *buttons, ButtonMode mode)
             gtk_tree_store_set(from_model, &parent, 0, categories[i].name, 1, i, -1);
             for (j = 0; categories[i].units[j].ui_name != NULL; j++) {
                 gtk_tree_store_append(from_model, &iter, &parent);
-                gtk_tree_store_set(from_model, &iter, 0, categories[i].units[j].ui_name, 1, j, -1);
+                gtk_tree_store_set(from_model, &iter, 0, categories[i].units[j].ui_name, 1, i, 2, j, -1);
             }
         }
         renderer = gtk_cell_renderer_text_new();
         gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(buttons->priv->convert_from_combo), renderer, TRUE);
         gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(buttons->priv->convert_from_combo), renderer, "text", 0);
+        gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(buttons->priv->convert_from_combo),
+                                           renderer,
+                                           convert_from_cell_data_func,
+                                           NULL, NULL);
 
         buttons->priv->convert_to_combo = GET_WIDGET(builder, "convert_to_combo");
         renderer = gtk_cell_renderer_text_new();
@@ -996,7 +1084,10 @@ load_mode(MathButtons *buttons, ButtonMode mode)
 
         g_signal_connect(buttons->priv->convert_from_combo, "changed", G_CALLBACK(convert_from_combobox_changed_cb), buttons);
         g_signal_connect(buttons->priv->convert_to_combo, "changed", G_CALLBACK(convert_to_combobox_changed_cb), buttons);
-        g_signal_connect(buttons->priv->equation, "answer-changed", G_CALLBACK(update_conversion_combos), buttons);
+        g_signal_connect(buttons->priv->equation, "notify::source-units", G_CALLBACK(source_units_changed_cb), buttons);
+        g_signal_connect(buttons->priv->equation, "notify::target-units", G_CALLBACK(target_units_changed_cb), buttons);
+        source_units_changed_cb(buttons->priv->equation, NULL, buttons);
+        target_units_changed_cb(buttons->priv->equation, NULL, buttons);
     }
 
     if (mode == PROGRAMMING) {
@@ -1056,6 +1147,8 @@ load_mode(MathButtons *buttons, ButtonMode mode)
         buttons->priv->target_currency_combo = GET_WIDGET(builder, "target_currency_combo");
         buttons->priv->currency_label = GET_WIDGET(builder, "currency_label");
 
+        buttons->priv->currency_serializer = mp_serializer_new(MP_DISPLAY_FORMAT_AUTOMATIC, 10, 2);
+
         model = gtk_list_store_new(1, G_TYPE_STRING);
 
         for (i = 0; currency_names[i].short_name != NULL; i++) {
@@ -1286,7 +1379,7 @@ make_register_menu_item(MathButtons *buttons, const gchar *name, const MPNumber
 
     if (value) {
         MpSerializer *serializer = math_equation_get_serializer(buttons->priv->equation);
-        mp_serializer_to_standard_string(serializer, value, &text);
+        text = mp_serializer_to_string(serializer, value);
         mstr = g_strdup_printf("<span weight=\"bold\">%s</span> = %s", name, text);
         g_free(text);
     }
@@ -1555,7 +1648,7 @@ function_cb(GtkWidget *widget, MathButtons *buttons)
             GtkWidget *item;
           
             item = gtk_menu_item_new_with_label(_(functions[i].name));
-            g_object_set_data(G_OBJECT(item), "function", g_strdup (functions[i].function));
+            g_object_set_data(G_OBJECT(item), "function", g_strdup(functions[i].function));
             gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
             g_signal_connect(item, "activate", G_CALLBACK(insert_function_cb), buttons);
             gtk_widget_show(item);
@@ -1571,7 +1664,7 @@ G_MODULE_EXPORT
 void
 factorize_cb(GtkWidget *widget, MathButtons *buttons)
 {
-    math_equation_factorize (buttons->priv->equation);
+    math_equation_factorize(buttons->priv->equation);
 }
 
 
@@ -1629,7 +1722,7 @@ finc_activate_cb(GtkWidget *widget, MathButtons *buttons)
     if (finc_dialog_fields[dialog][field+1] == NULL) {
         GtkWidget *dialog_widget;
         dialog_widget = gtk_widget_get_toplevel(widget);
-        if (gtk_widget_is_toplevel (dialog_widget)) {
+        if (gtk_widget_is_toplevel(dialog_widget)) {
             gtk_dialog_response(GTK_DIALOG(dialog_widget),
                                 GTK_RESPONSE_OK);
             return;
@@ -1656,7 +1749,7 @@ finc_response_cb(GtkWidget *widget, gint response_id, MathButtons *buttons)
     if (response_id != GTK_RESPONSE_OK)
         return;
 
-    dialog = GPOINTER_TO_INT (g_object_get_data(G_OBJECT(widget), "finc_dialog"));
+    dialog = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "finc_dialog"));
 
     for (i = 0; i < 4; i++) {
         if (finc_dialog_fields[dialog][i] == NULL) {
@@ -1732,6 +1825,21 @@ bit_toggle_cb(GtkWidget *event_box, GdkEventButton *event, MathButtons *buttons)
 }
 
 
+static void
+remove_trailing_spaces(MathButtons *buttons)
+{
+    GtkTextMark *insert_mark;
+    GtkTextIter start, end;
+    insert_mark = gtk_text_buffer_get_insert (GTK_TEXT_BUFFER(buttons->priv->equation));
+    gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(buttons->priv->equation), &end, insert_mark);
+    start = end;
+    while (gtk_text_iter_backward_char(&start)) {
+        if (!g_unichar_isspace(gtk_text_iter_get_char(&start)))
+            break;
+        gtk_text_buffer_delete(GTK_TEXT_BUFFER(buttons->priv->equation), &start, &end);
+    }
+}
+
 
 void set_superscript_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
@@ -1739,10 +1847,10 @@ void
 set_superscript_cb(GtkWidget *widget, MathButtons *buttons)
 {
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
-        char* display;
         math_equation_set_number_mode(buttons->priv->equation, SUPERSCRIPT);
-        display = math_equation_get_display(buttons->priv->equation);
-        math_equation_set(buttons->priv->equation, g_strchomp(display));
+        if (!gtk_text_buffer_get_has_selection(GTK_TEXT_BUFFER(buttons->priv->equation))) {
+            remove_trailing_spaces(buttons);
+        }
     }
     else if (math_equation_get_number_mode(buttons->priv->equation) == SUPERSCRIPT)
         math_equation_set_number_mode(buttons->priv->equation, NORMAL);
@@ -1755,10 +1863,10 @@ void
 set_subscript_cb(GtkWidget *widget, MathButtons *buttons)
 {
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
-        char* display;
         math_equation_set_number_mode(buttons->priv->equation, SUBSCRIPT);
-        display = math_equation_get_display(buttons->priv->equation);
-        math_equation_set(buttons->priv->equation, g_strchomp(display));
+        if (!gtk_text_buffer_get_has_selection(GTK_TEXT_BUFFER(buttons->priv->equation))) {
+            remove_trailing_spaces(buttons);
+        }
     }
     else if (math_equation_get_number_mode(buttons->priv->equation) == SUBSCRIPT)
         math_equation_set_number_mode(buttons->priv->equation, NORMAL);
@@ -1785,18 +1893,18 @@ number_mode_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons *bu
 
 
 static void
-math_buttons_set_property (GObject      *object,
-                           guint         prop_id,
-                           const GValue *value,
-                           GParamSpec   *pspec)
+math_buttons_set_property(GObject      *object,
+                          guint         prop_id,
+                          const GValue *value,
+                          GParamSpec   *pspec)
 {
     MathButtons *self;
 
-    self = MATH_BUTTONS (object);
+    self = MATH_BUTTONS(object);
 
     switch (prop_id) {
     case PROP_EQUATION:
-        self->priv->equation = g_value_get_object (value);
+        self->priv->equation = g_value_get_object(value);
         math_buttons_set_mode(self, self->priv->mode);
         g_signal_connect(self->priv->equation, "notify::display", G_CALLBACK(display_changed_cb), self);
         g_signal_connect(self->priv->equation, "notify::number-mode", G_CALLBACK(number_mode_changed_cb), self);
@@ -1806,41 +1914,41 @@ math_buttons_set_property (GObject      *object,
         display_changed_cb(self->priv->equation, NULL, self);
         break;
     case PROP_MODE:
-        math_buttons_set_mode(self, g_value_get_int (value));
+        math_buttons_set_mode(self, g_value_get_int(value));
         break;
     default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
     }
 }
 
 
 static void
-math_buttons_get_property (GObject    *object,
-                           guint       prop_id,
-                           GValue     *value,
-                           GParamSpec *pspec)
+math_buttons_get_property(GObject    *object,
+                          guint       prop_id,
+                          GValue     *value,
+                          GParamSpec *pspec)
 {
     MathButtons *self;
 
-    self = MATH_BUTTONS (object);
+    self = MATH_BUTTONS(object);
 
     switch (prop_id) {
     case PROP_EQUATION:
-        g_value_set_object (value, self->priv->equation);
+        g_value_set_object(value, self->priv->equation);
         break;
     case PROP_MODE:
-        g_value_set_int (value, self->priv->mode);
+        g_value_set_int(value, self->priv->mode);
         break;
     default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
     }
 }
 
 
 static void
-math_buttons_class_init (MathButtonsClass *klass)
+math_buttons_class_init(MathButtonsClass *klass)
 {
     static GEnumValue button_mode_values[] =
     {
@@ -1850,37 +1958,37 @@ math_buttons_class_init (MathButtonsClass *klass)
       {PROGRAMMING, "programming", "programming"},
       {0, NULL, NULL}
     };
-    GObjectClass *object_class = G_OBJECT_CLASS (klass);
+    GObjectClass *object_class = G_OBJECT_CLASS(klass);
 
     object_class->get_property = math_buttons_get_property;
     object_class->set_property = math_buttons_set_property;
 
-    g_type_class_add_private (klass, sizeof (MathButtonsPrivate));
+    g_type_class_add_private(klass, sizeof(MathButtonsPrivate));
 
     button_mode_type = g_enum_register_static("ButtonMode", button_mode_values);
 
-    g_object_class_install_property (object_class,
-                                     PROP_EQUATION,
-                                     g_param_spec_object ("equation",
-                                                          "equation",
-                                                          "Equation being controlled",
-                                                          math_equation_get_type(),
-                                                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-    g_object_class_install_property (object_class,
-                                     PROP_MODE,
-                                     g_param_spec_enum ("mode",
-                                                        "mode",
-                                                        "Button mode",
-                                                        button_mode_type,
-                                                        BASIC,
-                                                        G_PARAM_READWRITE));
+    g_object_class_install_property(object_class,
+                                    PROP_EQUATION,
+                                    g_param_spec_object("equation",
+                                                        "equation",
+                                                        "Equation being controlled",
+                                                        math_equation_get_type(),
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+    g_object_class_install_property(object_class,
+                                    PROP_MODE,
+                                    g_param_spec_enum("mode",
+                                                      "mode",
+                                                      "Button mode",
+                                                      button_mode_type,
+                                                      BASIC,
+                                                      G_PARAM_READWRITE));
 }
 
 
 static void
-math_buttons_init (MathButtons *buttons)
+math_buttons_init(MathButtons *buttons)
 {
-    buttons->priv = G_TYPE_INSTANCE_GET_PRIVATE (buttons, math_buttons_get_type(), MathButtonsPrivate);
+    buttons->priv = G_TYPE_INSTANCE_GET_PRIVATE(buttons, math_buttons_get_type(), MathButtonsPrivate);
     buttons->priv->programming_base = 10;
     gdk_color_parse("#0000FF", &buttons->priv->color_numbers);
     gdk_color_parse("#00FF00", &buttons->priv->color_action);
