@@ -49,6 +49,8 @@ enum {
     PROP_ANGLE_UNITS,
     PROP_SOURCE_CURRENCY,
     PROP_TARGET_CURRENCY,
+    PROP_SOURCE_UNITS,
+    PROP_TARGET_UNITS,  
     PROP_SERIALIZER
 };
 
@@ -76,15 +78,17 @@ struct MathEquationPrivate
     MPAngleUnit angle_units;  /* Units for trigonometric functions */
     char *source_currency;
     char *target_currency;
+    char *source_units;
+    char *target_units;
     NumberMode number_mode;   /* ??? */
     gboolean can_super_minus; /* TRUE if entering minus can generate a superscript minus */
 
-    const char *digits[16];   /* Localized digit values */
+    gunichar digits[16];      /* Localized digits */
 
     GtkTextMark *ans_start, *ans_end;
 
     MathEquationState state;  /* Equation state */
-    GList *undo_stack;           /* History of expression mode states */
+    GList *undo_stack;        /* History of expression mode states */
     GList *redo_stack;
     gboolean in_undo_operation;
   
@@ -112,7 +116,7 @@ G_DEFINE_TYPE (MathEquation, math_equation, GTK_TYPE_TEXT_BUFFER);
 MathEquation *
 math_equation_new()
 {
-    return g_object_new (math_equation_get_type(), NULL);
+    return g_object_new(math_equation_get_type(), NULL);
 }
 
 
@@ -153,7 +157,7 @@ reformat_ans(MathEquation *equation)
     gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(equation), &ans_start, equation->priv->ans_start);
     gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(equation), &ans_end, equation->priv->ans_end);
     orig_ans_text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(equation), &ans_start, &ans_end, FALSE);
-    mp_serializer_to_standard_string(equation->priv->serializer, &equation->priv->state.ans, &ans_text);
+    ans_text = mp_serializer_to_string(equation->priv->serializer, &equation->priv->state.ans);
     if (strcmp(orig_ans_text, ans_text) != 0) {
         gint start;
 
@@ -178,13 +182,120 @@ reformat_ans(MathEquation *equation)
     g_free(ans_text);
 }
 
+
+static gint
+count_digits(MathEquation *equation, const gchar *text)
+{
+    const gchar *read_iter;
+    gint count = 0;
+
+    read_iter = text;
+    while (*read_iter != '\0') {
+        if (!g_unichar_isdigit(g_utf8_get_char(read_iter)))
+            return count;
+
+        read_iter = g_utf8_next_char(read_iter);
+
+        /* Allow a thousands separator between digits follow a digit */
+        if (g_utf8_get_char(read_iter) == mp_serializer_get_thousands_separator(equation->priv->serializer)) {
+            read_iter = g_utf8_next_char(read_iter);
+            if (!g_unichar_isdigit(g_utf8_get_char(read_iter)))
+                return count;
+        }
+
+        count++;
+    }
+
+    return count;
+}
+
+
+static void
+reformat_separators(MathEquation *equation)
+{
+    gchar *text, *read_iter;
+    gint ans_start, ans_end;
+    gint offset, digit_offset = 0;
+    gboolean in_number = FALSE, in_radix = FALSE, last_is_tsep = FALSE;
+
+    equation->priv->in_undo_operation = TRUE;
+    equation->priv->in_reformat = TRUE;
+
+    text = math_equation_get_display(equation);
+    get_ans_offsets(equation, &ans_start, &ans_end);
+    for (read_iter = text, offset = 0; *read_iter != '\0'; read_iter = g_utf8_next_char(read_iter), offset++) {
+        gunichar c;
+        gboolean expect_tsep;
+
+        /* See what digit this character is */
+        c = g_utf8_get_char(read_iter);
+
+        expect_tsep = mp_serializer_get_show_thousands_separators(equation->priv->serializer) &&
+                      in_number && !in_radix && !last_is_tsep &&
+                      digit_offset > 0 && digit_offset % mp_serializer_get_thousands_separator_count(equation->priv->serializer) == 0;
+        last_is_tsep = FALSE;
+
+        /* Don't mess with ans */
+        if (offset >= ans_start && offset <= ans_end) {
+            in_number = in_radix = FALSE;
+            continue;
+        }
+        if (g_unichar_isdigit(c)) {
+            if (!in_number)
+                digit_offset = count_digits(equation, read_iter);
+            in_number = TRUE;
+
+            /* Expected a thousands separator between these digits - insert it */
+            if (expect_tsep) {
+                GtkTextIter iter;
+                gchar buffer[7];
+                gint len;
+
+                gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(equation), &iter, offset);
+                len = g_unichar_to_utf8(mp_serializer_get_thousands_separator(equation->priv->serializer), buffer);
+                buffer[len] = '\0';
+                gtk_text_buffer_insert(GTK_TEXT_BUFFER(equation), &iter, buffer, -1);
+                offset++;
+                last_is_tsep = TRUE;
+            }
+
+            digit_offset--;
+        }
+        else if (c == mp_serializer_get_radix(equation->priv->serializer)) {
+            in_number = in_radix = TRUE;
+        }
+        else if (c == mp_serializer_get_thousands_separator(equation->priv->serializer)) {
+            /* Didn't expect thousands separator - delete it */
+            if (!expect_tsep) {
+                GtkTextIter start, end;
+                gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(equation), &start, offset);
+                gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(equation), &end, offset + 1);
+                gtk_text_buffer_delete(GTK_TEXT_BUFFER(equation), &start, &end);
+                offset--;
+            }
+            else
+                last_is_tsep = TRUE;            
+        }
+        else {
+            in_number = in_radix = FALSE;
+        }
+    }
+
+    g_free(text);
+
+    equation->priv->in_reformat = FALSE;
+    equation->priv->in_undo_operation = FALSE;
+}
+
+
 static void
 reformat_display(MathEquation *equation, gint old_base)
 {
     /* Change ans */
     reformat_ans(equation);
 
-    g_signal_emit_by_name(equation, "answer-changed");
+    /* Add/remove thousands separators */
+    reformat_separators(equation);
 }
 
 
@@ -317,7 +428,7 @@ math_equation_copy(MathEquation *equation)
 
     text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(equation), &start, &end, FALSE);
     gtk_clipboard_set_text(gtk_clipboard_get(GDK_NONE), text, -1);
-    g_free (text);
+    g_free(text);
 }
 
 
@@ -326,7 +437,7 @@ on_paste(GtkClipboard *clipboard, const gchar *text, gpointer data)
 {
     MathEquation *equation = data;
     if (text != NULL)
-        math_equation_insert (equation, text);
+        math_equation_insert(equation, text);
 }
 
 
@@ -387,7 +498,7 @@ math_equation_redo(MathEquation *equation)
 }
 
 
-const gchar *
+gunichar
 math_equation_get_digit_text(MathEquation *equation, guint digit)
 {
     return equation->priv->digits[digit];
@@ -452,9 +563,9 @@ math_equation_get_show_trailing_zeroes(MathEquation *equation)
 
 
 void
-math_equation_set_number_format(MathEquation *equation, DisplayFormat format)
+math_equation_set_number_format(MathEquation *equation, MpDisplayFormat format)
 {
-    DisplayFormat old_format = mp_serializer_get_number_format(equation->priv->serializer);
+    MpDisplayFormat old_format = mp_serializer_get_number_format(equation->priv->serializer);
     if (old_format == format)
         return;
 
@@ -464,7 +575,7 @@ math_equation_set_number_format(MathEquation *equation, DisplayFormat format)
 }
 
 
-DisplayFormat
+MpDisplayFormat
 math_equation_get_number_format(MathEquation *equation)
 {
     return mp_serializer_get_number_format(equation->priv->serializer);
@@ -540,6 +651,7 @@ math_equation_set_source_currency(MathEquation *equation, const gchar *currency)
     g_object_notify(G_OBJECT(equation), "source-currency");
 }
 
+
 const gchar *
 math_equation_get_source_currency(MathEquation *equation)
 {
@@ -569,6 +681,41 @@ math_equation_get_target_currency(MathEquation *equation)
 }
 
 
+void
+math_equation_set_source_units(MathEquation *equation, const gchar *units)
+{
+    if (strcmp(equation->priv->source_units, units) == 0)
+        return;
+    g_free(equation->priv->source_units);
+    equation->priv->source_units = g_strdup(units);
+    g_object_notify(G_OBJECT(equation), "source-units");
+}
+
+const gchar *
+math_equation_get_source_units(MathEquation *equation)
+{
+    return equation->priv->source_units;
+}
+
+
+void
+math_equation_set_target_units(MathEquation *equation, const gchar *units)
+{
+    if (strcmp(equation->priv->target_units, units) == 0)
+        return;
+    g_free(equation->priv->target_units);
+    equation->priv->target_units = g_strdup(units);
+    g_object_notify(G_OBJECT(equation), "target-units");
+}
+
+
+const gchar *
+math_equation_get_target_units(MathEquation *equation)
+{
+    return equation->priv->target_units;
+}
+
+
 void
 math_equation_set_status(MathEquation *equation, const gchar *status)
 {
@@ -622,19 +769,52 @@ math_equation_get_display(MathEquation *equation)
 gchar *
 math_equation_get_equation(MathEquation *equation)
 {
-    char *text, *t;
-    gint ans_start, ans_end;
+    gchar *text;
+    GString *eq_text;
+    gint ans_start = -1, ans_end = -1, offset;
+    const gchar *read_iter;
+    gboolean last_is_digit = FALSE;
 
     text = math_equation_get_display(equation);
+    eq_text = g_string_sized_new(strlen(text));
 
-    /* No ans to substitute */
-    if(!equation->priv->ans_start)
-        return text;
+    if (equation->priv->ans_start)
+        get_ans_offsets(equation, &ans_start, &ans_end);
 
-    get_ans_offsets(equation, &ans_start, &ans_end);
-    t = g_strdup_printf("%.*sans%s", (int)(g_utf8_offset_to_pointer(text, ans_start) - text), text, g_utf8_offset_to_pointer(text, ans_end));
+    for (read_iter = text, offset = 0; *read_iter != '\0'; read_iter = g_utf8_next_char(read_iter), offset++) {
+        gunichar c;
+        gboolean is_digit, next_is_digit;
+
+        c = g_utf8_get_char(read_iter);
+        is_digit = g_unichar_isdigit(c);
+        next_is_digit = g_unichar_isdigit(g_utf8_get_char(g_utf8_next_char(read_iter)));
+
+        /* Replace ans text with variable */
+        if (offset == ans_start) { 
+             g_string_append(eq_text, "ans");
+             read_iter = g_utf8_offset_to_pointer(read_iter, ans_end - ans_start - 1);
+             offset += ans_end - ans_start - 1;
+             is_digit = FALSE;
+             continue;
+        }
+
+        /* Ignore thousands separators */
+        if (c == mp_serializer_get_thousands_separator(equation->priv->serializer) && last_is_digit && next_is_digit)
+            ;
+        /* Substitute radix character */
+        else if (c == mp_serializer_get_radix(equation->priv->serializer) && (last_is_digit || next_is_digit))
+            g_string_append_unichar(eq_text, '.');
+        else
+            g_string_append_unichar(eq_text, c);
+
+        last_is_digit = is_digit;
+    }
     g_free(text);
-    return t;
+
+    text = eq_text->str;
+    g_string_free(eq_text, FALSE);
+
+    return text;
 }
 
 
@@ -644,19 +824,21 @@ math_equation_get_number(MathEquation *equation, MPNumber *z)
     gchar *text;
     gboolean result;
 
-    text = math_equation_get_display(equation);
+    text = math_equation_get_equation(equation);
     result = !mp_serializer_from_string(equation->priv->serializer, text, z);
-    g_free (text);
+    g_free(text);
 
     return result;
 }
 
-MpSerializer*
+
+MpSerializer *
 math_equation_get_serializer(MathEquation *equation)
 {
     return equation->priv->serializer;
 }
 
+
 void
 math_equation_set_number_mode(MathEquation *equation, NumberMode mode)
 {
@@ -726,7 +908,7 @@ math_equation_set_number(MathEquation *equation, const MPNumber *x)
     GtkTextIter start, end;
 
     /* Show the number in the user chosen format */
-    mp_serializer_to_standard_string(equation->priv->serializer, x, &text);
+    text = mp_serializer_to_string(equation->priv->serializer, x);
     gtk_text_buffer_set_text(GTK_TEXT_BUFFER(equation), text, -1);
     mp_set_from_mp(x, &equation->priv->state.ans);
 
@@ -777,8 +959,13 @@ math_equation_insert_digit(MathEquation *equation, guint digit)
     static const char *subscript_digits[] = {"₀", "₁", "₂", "₃", "₄", "₅", "₆", "₇", "₈", "₉", NULL};
     static const char *superscript_digits[] = {"⁰", "¹", "²", "³", "⁴", "⁵", "⁶", "⁷", "⁸", "⁹", NULL};
 
-    if (equation->priv->number_mode == NORMAL || digit >= 10)
-        math_equation_insert(equation, math_equation_get_digit_text(equation, digit));
+    if (equation->priv->number_mode == NORMAL || digit >= 10) {
+        gchar buffer[7];
+        gint len;
+        len = g_unichar_to_utf8(math_equation_get_digit_text(equation, digit), buffer);
+        buffer[len] = '\0';
+        math_equation_insert(equation, buffer);
+    }
     else if (equation->priv->number_mode == SUPERSCRIPT)
         math_equation_insert(equation, superscript_digits[digit]);
     else if (equation->priv->number_mode == SUBSCRIPT)
@@ -789,7 +976,11 @@ math_equation_insert_digit(MathEquation *equation, guint digit)
 void
 math_equation_insert_numeric_point(MathEquation *equation)
 {
-    math_equation_insert(equation, mp_serializer_get_numeric_point_text(equation->priv->serializer));
+    gchar buffer[7];
+    gint len;
+    len = g_unichar_to_utf8( mp_serializer_get_radix(equation->priv->serializer), buffer);
+    buffer[len] = '\0';
+    math_equation_insert(equation, buffer);
 }
 
 
@@ -797,7 +988,7 @@ void
 math_equation_insert_number(MathEquation *equation, const MPNumber *x)
 {
     char *text;
-    mp_serializer_to_standard_string(equation->priv->serializer, x, &text);
+    text = mp_serializer_to_string(equation->priv->serializer, x);
     math_equation_insert(equation, text);
     g_free(text);
 }
@@ -837,10 +1028,10 @@ variable_is_defined(const char *name, void *data)
 
     if (strcmp(lower_name, "rand") == 0 || 
         strcmp(lower_name, "ans") == 0) {
-        g_free (lower_name);
+        g_free(lower_name);
         return 1;
     }
-    g_free (lower_name);
+    g_free(lower_name);
 
     return math_variables_get_value(equation->priv->variables, name) != NULL;
 }
@@ -910,6 +1101,7 @@ parse(MathEquation *equation, const char *text, MPNumber *z, char **error_token)
     return mp_equation_parse(text, &options, z, error_token);
 }
 
+
 /*
  * Executed in separate thread. It is thus not a good idea to write to anything
  * in MathEquation but the async queue from here.
@@ -984,6 +1176,7 @@ math_equation_solve_real(gpointer data)
     return NULL;
 }
 
+
 static gboolean
 math_equation_show_in_progress(gpointer data)
 {
@@ -993,6 +1186,7 @@ math_equation_show_in_progress(gpointer data)
     return false;
 }
 
+
 static gboolean
 math_equation_look_for_answer(gpointer data)
 {
@@ -1021,10 +1215,10 @@ math_equation_look_for_answer(gpointer data)
     }
     g_slice_free(SolveData, result);
 
-    g_signal_emit_by_name(equation, "answer-changed");
     return false;
 }
 
+
 void
 math_equation_solve(MathEquation *equation)
 {
@@ -1077,7 +1271,7 @@ math_equation_factorize_real(gpointer data)
         MPNumber *n;
 
         n = factor->data;
-        mp_serializer_to_standard_string(equation->priv->serializer, n, &temp);
+        temp = mp_serializer_to_string(equation->priv->serializer, n);
         g_string_append(text, temp);
         if (factor->next)
             g_string_append(text, "×");
@@ -1093,6 +1287,7 @@ math_equation_factorize_real(gpointer data)
     return NULL;
 }
 
+
 void
 math_equation_factorize(MathEquation *equation)
 {
@@ -1222,7 +1417,7 @@ math_equation_set_property(GObject      *object,
 {
     MathEquation *self;
 
-    self = MATH_EQUATION (object);
+    self = MATH_EQUATION(object);
 
     switch (prop_id) {
     case PROP_STATUS:
@@ -1261,8 +1456,14 @@ math_equation_set_property(GObject      *object,
     case PROP_TARGET_CURRENCY:
         math_equation_set_target_currency(self, g_value_get_string(value));
         break;
+    case PROP_SOURCE_UNITS:
+        math_equation_set_source_units(self, g_value_get_string(value));
+        break;
+    case PROP_TARGET_UNITS:
+        math_equation_set_target_units(self, g_value_get_string(value));
+        break;
     default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
     }
 }
@@ -1277,7 +1478,7 @@ math_equation_get_property(GObject    *object,
     MathEquation *self;
     gchar *text;
 
-    self = MATH_EQUATION (object);
+    self = MATH_EQUATION(object);
 
     switch (prop_id) {
     case PROP_STATUS:
@@ -1323,17 +1524,24 @@ math_equation_get_property(GObject    *object,
     case PROP_TARGET_CURRENCY:
         g_value_set_string(value, self->priv->target_currency);
         break;
+    case PROP_SOURCE_UNITS:
+        g_value_set_string(value, self->priv->source_units);
+        break;
+    case PROP_TARGET_UNITS:
+        g_value_set_string(value, self->priv->target_units);
+        break;
     case PROP_SERIALIZER:
         g_value_set_object(value, self->priv->serializer);
         break;
     default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
     }
 }
 
+
 static void
-math_equation_constructed (GObject *object)
+math_equation_constructed(GObject *object)
 {
     GtkTextBuffer *parent_class;
     parent_class = g_type_class_peek_parent(MATH_EQUATION_GET_CLASS(object));
@@ -1344,9 +1552,8 @@ math_equation_constructed (GObject *object)
 }
 
 
-
 static void
-math_equation_class_init (MathEquationClass *klass)
+math_equation_class_init(MathEquationClass *klass)
 {
     static GEnumValue number_mode_values[] =
     {
@@ -1362,16 +1569,16 @@ math_equation_class_init (MathEquationClass *klass)
       {MP_GRADIANS, "gradians", "gradians"},
       {0, NULL, NULL}
     };
-    GObjectClass *object_class = G_OBJECT_CLASS (klass);
+    GObjectClass *object_class = G_OBJECT_CLASS(klass);
 
     object_class->get_property = math_equation_get_property;
     object_class->set_property = math_equation_set_property;
     object_class->constructed = math_equation_constructed;
 
-    g_type_class_add_private (klass, sizeof (MathEquationPrivate));
+    g_type_class_add_private(klass, sizeof(MathEquationPrivate));
   
     number_mode_type = g_enum_register_static("NumberMode", number_mode_values);
-    number_format_type = math_display_format_get_type();
+    number_format_type = math_mp_display_format_get_type();
     angle_unit_type = g_enum_register_static("AngleUnit", angle_unit_values);
 
     g_object_class_install_property(object_class,
@@ -1430,7 +1637,7 @@ math_equation_class_init (MathEquationClass *klass)
                                                       "number-format",
                                                       "Display format",
                                                       number_format_type,
-                                                      FIX,
+                                                      MP_DISPLAY_FORMAT_FIXED,
                                                       G_PARAM_READWRITE));
     g_object_class_install_property(object_class,
                                     PROP_BASE,
@@ -1468,6 +1675,20 @@ math_equation_class_init (MathEquationClass *klass)
                                                         "target Currency",
                                                         "",
                                                         G_PARAM_READWRITE));
+    g_object_class_install_property(object_class,
+                                    PROP_SOURCE_UNITS,
+                                    g_param_spec_string("source-units",
+                                                        "source-units",
+                                                        "Source Units",
+                                                        "",
+                                                        G_PARAM_READWRITE));
+    g_object_class_install_property(object_class,
+                                    PROP_TARGET_UNITS,
+                                    g_param_spec_string("target-units",
+                                                        "target-units",
+                                                        "target Units",
+                                                        "",
+                                                        G_PARAM_READWRITE));
     g_object_class_install_property(object_class,
                                     PROP_SERIALIZER,
                                     g_param_spec_object("serializer",
@@ -1475,25 +1696,15 @@ math_equation_class_init (MathEquationClass *klass)
                                                         "Serializer",
                                                         MP_TYPE_SERIALIZER,
                                                         G_PARAM_READABLE));
-
-    g_signal_new ("answer-changed",
-                  G_TYPE_FROM_CLASS(object_class),
-                  G_SIGNAL_RUN_FIRST,
-                  0,
-                  NULL,
-                  NULL,
-                  g_cclosure_marshal_VOID__VOID,
-                  G_TYPE_NONE,
-                  0);
 }
 
 
 static void
-pre_insert_text_cb (MathEquation  *equation,
-                    GtkTextIter   *location,
-                    gchar         *text,
-                    gint           len,
-                    gpointer       user_data)
+pre_insert_text_cb(MathEquation  *equation,
+                   GtkTextIter   *location,
+                   gchar         *text,
+                   gint           len,
+                   gpointer       user_data)
 {
     gunichar c;
   
@@ -1508,8 +1719,8 @@ pre_insert_text_cb (MathEquation  *equation,
     /* Clear result on next digit entered if cursor at end of line */
     // FIXME Cursor
     c = g_utf8_get_char(text);
-    if ((g_unichar_isdigit(c) || mp_serializer_is_numeric_point(equation->priv->serializer, text)) &&
-            math_equation_is_result(equation)) {
+    if ((g_unichar_isdigit(c) || c == mp_serializer_get_radix(equation->priv->serializer)) &&
+         math_equation_is_result(equation)) {
         gtk_text_buffer_set_text(GTK_TEXT_BUFFER(equation), "", -1);
         clear_ans(equation, FALSE);
         gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(equation), location);
@@ -1532,16 +1743,16 @@ pre_insert_text_cb (MathEquation  *equation,
 static gboolean
 on_delete(MathEquation *equation)
 {
-  equation->priv->in_delete = FALSE;  
-  return FALSE;
+    equation->priv->in_delete = FALSE;  
+    return FALSE;
 }
 
 
 static void
-pre_delete_range_cb (MathEquation  *equation,
-                     GtkTextIter   *start,
-                     GtkTextIter   *end,
-                     gpointer       user_data)
+pre_delete_range_cb(MathEquation  *equation,
+                    GtkTextIter   *start,
+                    GtkTextIter   *end,
+                    gpointer       user_data)
 {  
     if (equation->priv->in_reformat)
         return;
@@ -1567,30 +1778,38 @@ pre_delete_range_cb (MathEquation  *equation,
 
 
 static void
-insert_text_cb (MathEquation  *equation,
-                GtkTextIter   *location,
-                gchar         *text,
-                gint           len,
-                gpointer       user_data)
+insert_text_cb(MathEquation  *equation,
+               GtkTextIter   *location,
+               gchar         *text,
+               gint           len,
+               gpointer       user_data)
 {
     if (equation->priv->in_reformat)
         return;
 
     equation->priv->state.entered_multiply = strcmp(text, "×") == 0;
+
+    /* Update thousands separators */
+    reformat_separators(equation);
+
     g_object_notify(G_OBJECT(equation), "display");
 }
 
 
 static void
-delete_range_cb (MathEquation  *equation,
-                 GtkTextIter   *start,
-                 GtkTextIter   *end,
-                 gpointer       user_data)
+delete_range_cb(MathEquation  *equation,
+                GtkTextIter   *start,
+                GtkTextIter   *end,
+                gpointer       user_data)
 {
     if (equation->priv->in_reformat)
         return;
 
     equation->priv->state.entered_multiply = FALSE;
+
+    /* Update thousands separators */
+    reformat_separators(equation);
+
     // FIXME: A replace will emit this both for delete-range and insert-text, can it be avoided?
     g_object_notify(G_OBJECT(equation), "display");
 }
@@ -1606,7 +1825,7 @@ math_equation_init(MathEquation *equation)
     gboolean use_default_digits = FALSE;
     int i;
 
-    equation->priv = G_TYPE_INSTANCE_GET_PRIVATE (equation, math_equation_get_type(), MathEquationPrivate);
+    equation->priv = G_TYPE_INSTANCE_GET_PRIVATE(equation, math_equation_get_type(), MathEquationPrivate);
 
     g_signal_connect(equation, "insert-text", G_CALLBACK(pre_insert_text_cb), equation);
     g_signal_connect(equation, "delete-range", G_CALLBACK(pre_delete_range_cb), equation);  
@@ -1617,10 +1836,10 @@ math_equation_init(MathEquation *equation)
     for (i = 0; i < 16; i++) {
         if (use_default_digits || digits[i] == NULL) {
             use_default_digits = TRUE;
-            equation->priv->digits[i] = strdup(default_digits[i]);
+            equation->priv->digits[i] = g_utf8_get_char(default_digits[i]);
         }
         else
-            equation->priv->digits[i] = strdup(digits[i]);
+            equation->priv->digits[i] = g_utf8_get_char(digits[i]);
     }
     g_strfreev(digits);
 
@@ -1632,7 +1851,9 @@ math_equation_init(MathEquation *equation)
     // FIXME: Pick based on locale
     equation->priv->source_currency = g_strdup(currency_names[0].short_name);
     equation->priv->target_currency = g_strdup(currency_names[0].short_name);
-    equation->priv->serializer = mp_serializer_new();
+    equation->priv->source_units = g_strdup("");
+    equation->priv->target_units = g_strdup("");
+    equation->priv->serializer = mp_serializer_new(MP_DISPLAY_FORMAT_AUTOMATIC, 10, 9);
     equation->priv->queue = g_async_queue_new();
 
     mp_set_from_integer(0, &equation->priv->state.ans);
