@@ -56,9 +56,7 @@ MathEquation *math_equation_new(void);
 
 MathVariables *math_equation_get_variables(MathEquation *equation);
 
-const gchar *math_equation_get_digit_text(MathEquation *equation, guint digit);
-const gchar *math_equation_get_numeric_point_text(MathEquation *equation);
-const gchar *math_equation_get_thousands_separator_text(MathEquation *equation);
+gunichar math_equation_get_digit_text(MathEquation *equation, guint digit);
 
 void math_equation_set_status(MathEquation *equation, const gchar *status);
 const gchar *math_equation_get_status(MathEquation *equation);
@@ -81,8 +79,8 @@ gboolean math_equation_get_show_thousands_separators(MathEquation *equation);
 void math_equation_set_show_trailing_zeroes(MathEquation *equation, gboolean visible);
 gboolean math_equation_get_show_trailing_zeroes(MathEquation *equation);
 
-void math_equation_set_number_format(MathEquation *equation, DisplayFormat format);
-DisplayFormat math_equation_get_number_format(MathEquation *equation);
+void math_equation_set_number_format(MathEquation *equation, MpDisplayFormat format);
+MpDisplayFormat math_equation_get_number_format(MathEquation *equation);
 
 void math_equation_set_base(MathEquation *equation, gint base);
 gint math_equation_get_base(MathEquation *equation);
@@ -99,6 +97,12 @@ const gchar *math_equation_get_source_currency(MathEquation *equation);
 void math_equation_set_target_currency(MathEquation *equation, const gchar *currency);
 const gchar *math_equation_get_target_currency(MathEquation *equation);
 
+void math_equation_set_source_units(MathEquation *equation, const gchar *units);
+const gchar *math_equation_get_source_units(MathEquation *equation);
+
+void math_equation_set_target_units(MathEquation *equation, const gchar *units);
+const gchar *math_equation_get_target_units(MathEquation *equation);
+
 gboolean math_equation_in_solve(MathEquation *equation);
 
 const MPNumber *math_equation_get_answer(MathEquation *equation);
